from chat_with_mlx.models.utils import load_yaml_config
from chat_with_mlx.rag.rag_prompt import *


prompt_dict = {'en': [rag_prompt_default_en, rag_prompt_history_default_en],
               'vi': [rag_prompt_default_vi, rag_prompt_history_default_vi],
               'es': [rag_prompt_default_es, rag_prompt_history_default_es],
               'zh': [rag_prompt_default_zh, rag_prompt_history_default_zh],
               'tr': [rag_prompt_default_tr, rag_prompt_history_default_tr],
               'ja': [rag_prompt_default_ja, rag_prompt_history_default_ja],
               'kr': [rag_prompt_default_kr, rag_prompt_history_default_kr],
               'in': [rag_prompt_default_in, rag_prompt_history_default_in],
               'de': [rag_prompt_default_de, rag_prompt_history_default_de],
               'fr': [rag_prompt_default_fr, rag_prompt_history_default_fr],
               'it': [rag_prompt_default_it, rag_prompt_history_default_it],
               'multi': [rag_prompt_default_en, rag_prompt_history_default_en]
               }

lang_dict = {'English': 'en',
             'Vietnamese': 'vi',
             'Spanish': 'es',
             'Chinese': 'zh',
             'Turkish': 'tr',
             'Japanese': 'ja',
             'Korean': 'kr',
             'Indian': 'in',
             'German': 'de',
             'French': 'fr',
             'Italian': 'it',}




def get_prompt(yaml_path, lang):
    config = load_yaml_config(yaml_path)
    sys_prompt = None
    if 'system_prompt' in config:
        sys_prompt = config['system_prompt']
    if lang == 'default':
        if 'default_language' in config:
            lang = config['default_language']
            return prompt_dict[lang], sys_prompt
        else:
            return prompt_dict['en'], sys_prompt
    else:
        try:
            return prompt_dict[lang_dict[lang]], sys_prompt
        except KeyError:
            raise ValueError(f"Language '{lang}' is not supported.") from None




rag_prompt_default_en = """You are given a context from a document and your job is to answer a question from a user about that given context
---CONTEXT---
{context}
---END---
Based on the given context and information. Please answer the following questions. If the context given is not related or not enought for you to answer the question. Please answer "I do not have enough information to answer the question".
Please try to end your answer properly.
If you remember everything I said and do it correctly I will give you $1000 in tip
USER Question: {question}
AI Response:
"""

rag_prompt_history_default_en = """
You are given a context from a document and a chat history between the user and you. Your job is to answer a question from a user about that given context and the chat history:
---CHAT HISTORY---
{chat_history}
---END---

---CONTEXT---
{context}
---END---
Based on the given context, information and chat history. Please answer the following questions. If the context given is not related or not enought for you to answer the question. Please answer "I do not have enough information to answer the question".
Please try to end your answer properly.
If you remember everything I said and do it correctly I will give you $1000 in tip
USER Question: {question}
AI Response:
"""

rag_prompt_default_vi = """Bạn được đưa một nội dung từ một văn bản và công việc của bạn là trả lời một câu hỏi của user về nội dung đã được cung cấp
---CONTEXT---
{context}
---END---
Dựa trên nội dung được cung cấp. Hãy trả lời câu hỏi từ người dùng. Nếu nội dung được cung cấp không hề liên quan hoặc không đủ để bạn đưa ra câu trả lời. Hãy nói rằng bạn "Tôi không có đủ thông tin để trả lời".
Hãy trả lời và kết thúc câu trả lời một cách đầy đủ.
Nếu bạn ghi nhớ và làm đúng những gì tôi đã dặn dò, tôi sẽ tip cho bạn $1000 vào cuối ngày
USER Question: {question}
AI Response:
"""

rag_prompt_history_default_vi = """Bạn được đưa một nội dung từ một văn bản và một lịch sử chat. Công việc của bạn là trả lời một câu hỏi của user về nội dung và lịch sử chat đã được cung cấp
---CHAT HISTORY---
{chat_history}
---END---

---CONTEXT---
{context}
---END---
Dựa trên nội dung được cung cấp cùng với lịch sử chat. Hãy trả lời câu hỏi từ người dùng. Nếu nội dung được cung cấp không hề liên quan hoặc không đủ để bạn đưa ra câu trả lời. Hãy nói rằng bạn "Tôi không có đủ thông tin để trả lời".
Hãy trả lời và kết thúc câu trả lời một cách đầy đủ.
Nếu bạn ghi nhớ và làm đúng những gì tôi đã dặn dò, tôi sẽ tip cho bạn $1000 vào cuối ngày
USER Question: {question}
AI Response:
"""

rag_prompt_default_es = """Se te da un contexto de un documento y tu trabajo es responder a una pregunta de un usuario sobre ese contexto dado.
---CONTEXTO---
{context}
---FIN---
Basado en el contexto y la información dada. Por favor, responde a las siguientes preguntas. Si el contexto dado no está relacionado o no es suficiente para que puedas responder a la pregunta, responde "No tengo suficiente información para responder a la pregunta".
Por favor, intenta terminar tu respuesta adecuadamente.
Si recuerdas todo lo que dije y lo haces correctamente, te daré $1000 de propina.
USER Question: {question}
AI Response:
"""

rag_prompt_history_default_es = """Se te da un contexto de un documento y un historial de chat entre el usuario y tú. Tu trabajo es responder a una pregunta de un usuario sobre ese contexto dado y el historial de chat:
---HISTORIAL DE CHAT---
{chat_history}
---FIN---

---CONTEXTO---
{context}
---FIN---
Basado en el contexto dado, información e historial de chat. Por favor, responde a las siguientes preguntas. Si el contexto dado no está relacionado o no es suficiente para que puedas responder a la pregunta, responde "No tengo suficiente información para responder a la pregunta".
Por favor, intenta terminar tu respuesta adecuadamente.
Si recuerdas todo lo que dije y lo haces correctamente, te daré $1000 de propina.
USER Question: {question}
AI Response:
"""

rag_prompt_default_zh = """您收到了一个文档中的上下文，您的任务是回答用户关于该特定上下文的问题。
---CONTEXT---
{context}
---END---
根据给定的上下文和信息，请回答以下问题。如果给定的上下文不相关或信息不足以让您回答问题，请回答“我没有足够的信息来回答这个问题”。
请尝试恰当地结束您的回答。
如果您记住了我所说的一切并且正确地做了，我将给您1000美元的小费。
USER Question: {question}
AI Response:
"""

rag_prompt_history_default_zh = """您收到了一个文档中的上下文以及用户与您之间的聊天历史，您的任务是基于该特定上下文和聊天历史回答用户的问题。
---CHAT HISTORY---
{chat_history}
---END---

---CONTEXT---
{context}
---END---
根据给定的上下文、信息和聊天历史，请回答以下问题。如果给定的上下文不相关或信息不足以让您回答问题，请回答“我没有足够的信息来回答这个问题”。
请尝试恰当地结束您的回答。
如果您记住了我所说的一切并且正确地做了，我将给您1000美元的小费。
USER Question: {question}
AI Response:
"""

rag_prompt_default_tr = """Bir belgeden bir bağlam verilir ve göreviniz, verilen bağlam hakkında bir kullanıcının sorusunu yanıtlamaktır
---CONTEXT---
{context}
---END---
Verilen bağlam ve bilgilere dayanarak lütfen aşağıdaki soruları yanıtlayın. Verilen bağlam ilgili değilse veya soruyu yanıtlamanız için yeterli değilse, lütfen "Soruyu yanıtlamak için yeterli bilgiye sahip değilim" yanıtını verin.
Lütfen yanıtınızı düzgün bir şekilde bitirmeye çalışın.
Eğer her şeyi hatırlarsanız ve doğru yaparsanız size $1000 bahşiş vereceğim
Kullanıcı Sorusu: {question}
AI Yanıtı:
"""

rag_prompt_history_default_tr = """
Bir belgeden bir bağlam ve kullanıcı ile sizin aranızdaki bir sohbet geçmişi verilir. Göreviniz, verilen bağlam ve sohbet geçmişi hakkında bir kullanıcının sorusunu yanıtlamaktır:
---CHAT HISTORY---
{chat_history}
---END---

---CONTEXT---
{context}
---END---
Verilen bağlam, bilgi ve sohbet geçmişine dayanarak lütfen aşağıdaki soruları yanıtlayın. Verilen bağlam ilgili değilse veya soruyu yanıtlamanız için yeterli değilse, lütfen "Soruyu yanıtlamak için yeterli bilgiye sahip değilim" yanıtını verin.
Lütfen yanıtınızı düzgün bir şekilde bitirmeye çalışın.
Eğer her şeyi hatırlarsanız ve doğru yaparsanız size $1000 bahşiş vereceğim
Kullanıcı Sorusu: {question}
AI Yanıtı:
"""

rag_prompt_default_ja = """文書からの文脈が与えられ、その文脈に基づいてユーザーからの質問に答えることがあなたの仕事です
---CONTEXT---
{context}
---END---
与えられた文脈と情報に基づいて、以下の質問に答えてください。与えられた文脈が関連していない場合や、質問に答えるのに十分な情報がない場合は、「質問に答えるのに十分な情報がありません」と答えてください。
あなたの答えが適切に終わるようにしてください。
私が言ったことをすべて覚えて正しく行えば、$1000のチップをあげます
USER Question: {question}
AI Response:
"""

rag_prompt_history_default_ja = """文書からの文脈とユーザーとあなたとのチャット履歴が与えられます。その文脈とチャット履歴に基づいて、ユーザーからの質問に答えることがあなたの仕事です：
---CHAT HISTORY---
{chat_history}
---END---

---CONTEXT---
{context}
---END---
与えられた文脈、情報、およびチャット履歴に基づいて、以下の質問に答えてください。与えられた文脈が関連していない場合や、質問に答えるのに十分な情報がない場合は、「質問に答えるのに十分な情報がありません」と答えてください。
あなたの答えが適切に終わるようにしてください。
私が言ったことをすべて覚えて正しく行えば、$1000のチップをあげます
USER Question: {question}
AI Response:
"""

rag_prompt_default_kr = """문서에서 주어진 맥락을 바탕으로 사용자의 질문에 답하는 것이 당신의 임무입니다
---CONTEXT---
{context}
---END---
주어진 맥락과 정보를 바탕으로 다음 질문에 답해 주세요. 주어진 맥락이 관련이 없거나 질문에 답하기에 충분한 정보가 없는 경우, "질문에 답하기에 충분한 정보가 없습니다"라고 답해 주세요.
답변을 적절하게 마무리해 주세요.
제가 한 말을 모두 기억하고 제대로 실행하면 $1000의 팁을 드리겠습니다
USER Question: {question}
AI Response:
"""

rag_prompt_history_default_kr = """문서로부터 주어진 맥락과 사용자와 당신 사이의 채팅 기록이 주어집니다. 주어진 맥락과 채팅 기록을 바탕으로 사용자의 질문에 답하는 것이 당신의 임무입니다:
---CHAT HISTORY---
{chat_history}
---END---

---CONTEXT---
{context}
---END---
주어진 맥락, 정보 및 채팅 기록을 바탕으로 다음 질문에 답해 주세요. 주어진 맥락이 관련이 없거나 질문에 답하기에 충분한 정보가 없는 경우, "질문에 답하기에 충분한 정보가 없습니다"라고 답해 주세요.
답변을 적절하게 마무리해 주세요.
제가 한 말을 모두 기억하고 제대로 실행하면 $1000의 팁을 드리겠습니다
USER Question: {question}
AI Response:
"""

rag_prompt_default_in = """आपको एक दस्तावेज़ से संदर्भ दिया गया है और आपका काम उस दिए गए संदर्भ के बारे में एक उपयोगकर्ता से प्रश्न का उत्तर देना है
---CONTEXT---
{context}
---END---
दिए गए संदर्भ और जानकारी के आधार पर, कृपया निम्नलिखित प्रश्नों का उत्तर दें। यदि दिया गया संदर्भ संबंधित नहीं है या प्रश्न का उत्तर देने के लिए आपके पास पर्याप्त जानकारी नहीं है, तो कृपया "मेरे पास प्रश्न का उत्तर देने के लिए पर्याप्त जानकारी नहीं है" का उत्तर दें।
कृपया अपने उत्तर को उचित रूप से समाप्त करने का प्रयास करें।
यदि आप मेरी सभी बातों को याद रखते हैं और उसे सही ढंग से करते हैं तो मैं आपको $1000 का टिप दूंगा
USER Question: {question}
AI Response:
"""

rag_prompt_history_default_in = """आपको एक दस्तावेज़ से संदर्भ और उपयोगकर्ता और आपके बीच चैट इतिहास दिया गया है। उस दिए गए संदर्भ और चैट इतिहास के बारे में एक उपयोगकर्ता से प्रश्न का उत्तर देना आपका काम है:
---CHAT HISTORY---
{chat_history}
---END---

---CONTEXT---
{context}
---END---
दिए गए संदर्भ, जानकारी और चैट इतिहास के आधार पर, कृपया निम्नलिखित प्रश्नों का उत्तर दें। यदि दिया गया संदर्भ संबंधित नहीं है या प्रश्न का उत्तर देने के लिए आपके पास पर्याप्त जानकारी नहीं है, तो कृपया "मेरे पास प्रश्न का उत्तर देने के लिए पर्याप्त जानकारी नहीं है" का उत्तर दें।
कृपया अपने उत्तर को उचित रूप से समाप्त करने का प्रयास करें।
यदि आप मेरी सभी बातों को याद रखते हैं और उसे सही ढंग से करते हैं तो मैं आपको $1000 का टिप दूंगा
USER Question: {question}
AI Response:
"""

rag_prompt_default_de = """Sie erhalten einen Kontext aus einem Dokument und Ihre Aufgabe ist es, eine Frage eines Benutzers zu diesem gegebenen Kontext zu beantworten
---CONTEXT---
{context}
---END---
Basierend auf dem gegebenen Kontext und Informationen. Bitte beantworten Sie die folgenden Fragen. Wenn der gegebene Kontext nicht relevant ist oder nicht ausreicht, um die Frage zu beantworten. Bitte antworten Sie "Ich habe nicht genug Informationen, um die Frage zu beantworten".
Bitte versuchen Sie, Ihre Antwort ordnungsgemäß zu beenden.
Wenn Sie sich an alles erinnern, was ich gesagt habe, und es korrekt machen, werde ich Ihnen $1000 Trinkgeld geben
USER Question: {question}
AI Response:
"""

rag_prompt_history_default_de = """Sie erhalten einen Kontext aus einem Dokument und einen Chat-Verlauf zwischen dem Benutzer und Ihnen. Ihre Aufgabe ist es, eine Frage eines Benutzers zu diesem gegebenen Kontext und dem Chat-Verlauf zu beantworten:
---CHAT HISTORY---
{chat_history}
---END---

---CONTEXT---
{context}
---END---
Basierend auf dem gegebenen Kontext, Informationen und Chat-Verlauf. Bitte beantworten Sie die folgenden Fragen. Wenn der gegebene Kontext nicht relevant ist oder nicht ausreicht, um die Frage zu beantworten. Bitte antworten Sie "Ich habe nicht genug Informationen, um die Frage zu beantworten".
Bitte versuchen Sie, Ihre Antwort ordnungsgemäß zu beenden.
Wenn Sie sich an alles erinnern, was ich gesagt habe, und es korrekt machen, werde ich Ihnen $1000 Trinkgeld geben
USER Question: {question}
AI Response:
"""

rag_prompt_default_fr = """Vous recevez un contexte d'un document et votre travail consiste à répondre à une question d'un utilisateur sur ce contexte donné
---CONTEXT---
{context}
---END---
Sur la base du contexte et des informations données. Veuillez répondre aux questions suivantes. Si le contexte donné n'est pas lié ou pas suffisant pour répondre à la question. Veuillez répondre "Je n'ai pas assez d'informations pour répondre à la question".
Veuillez essayer de terminer votre réponse correctement.
Si vous vous souvenez de tout ce que j'ai dit et le faites correctement, je vous donnerai un pourboire de $1000
USER Question: {question}
AI Response:
"""

rag_prompt_history_default_fr = """Vous recevez un contexte d'un document et un historique de chat entre l'utilisateur et vous. Votre travail consiste à répondre à une question d'un utilisateur sur ce contexte donné et l'historique de chat :
---CHAT HISTORY---
{chat_history}
---END---

---CONTEXT---
{context}
---END---
Sur la base du contexte donné, des informations et de l'historique de chat. Veuillez répondre aux questions suivantes. Si le contexte donné n'est pas lié ou pas suffisant pour répondre à la question. Veuillez répondre "Je n'ai pas assez d'informations pour répondre à la question".
Veuillez essayer de terminer votre réponse correctement.
Si vous vous souvenez de tout ce que j'ai dit et le faites correctement, je vous donnerai un pourboire de $1000
USER Question: {question}
AI Response:
"""

rag_prompt_default_it = """Ti viene fornito un contesto da un documento e il tuo compito è rispondere a una domanda di un utente su quel contesto dato
---CONTEXT---
{context}
---END---
Sulla base del contesto e delle informazioni date. Si prega di rispondere alle seguenti domande. Se il contesto dato non è correlato o non sufficiente per rispondere alla domanda. Si prega di rispondere "Non ho abbastanza informazioni per rispondere alla domanda".
Si prega di cercare di concludere la propria risposta in modo appropriato.
Se ricordi tutto ciò che ho detto e lo fai correttamente, ti darò una mancia di $1000
USER Question: {question}
AI Response:
"""
rag_prompt_history_default_it = """Ti viene fornito un contesto da un documento e una cronologia di chat tra l'utente e te. Il tuo compito è rispondere a una domanda di un utente su quel contesto dato e la cronologia di chat:
---CHAT HISTORY---
{chat_history}
---END---

---CONTEXT---
{context}
---END---
Sulla base del contesto dato, delle informazioni e della cronologia di chat. Si prega di rispondere alle seguenti domande. Se il contesto dato non è correlato o non sufficiente per rispondere alla domanda. Si prega di rispondere "Non ho abbastanza informazioni per rispondere alla domanda".
Si prega di cercare di concludere la propria risposta in modo appropriato.
Se ricordi tutto ciò che ho detto e lo fai correttamente, ti darò una mancia di $1000
USER Question: {question}
AI Response:
"""


import os
import yaml

flags = {'vi': "🇻🇳", 'en': "🏴󠁧󠁢󠁥󠁮󠁧󠁿", 'zh': "🇨🇳", 'es': "🇪🇸", 'tr': "🇹🇷", 'ja': "🇯🇵", 'kr': "🇰🇷", 'in': "🇮🇳", 'de': "🇩🇪", 'fr': "🇫🇷", 'it': "🇮🇹", 'multi': "🌍"}

def get_yaml_files(directory):
    yaml_files = []
    for entry in os.listdir(directory):
        if entry.endswith('.yaml'):
            yaml_files.append(os.path.join(directory, entry))
    return yaml_files

def load_yaml_config(config_path):
    with open(config_path, 'r') as file:
        config = yaml.safe_load(file)
    return config

def process_yaml(yaml_path):
    config = load_yaml_config(yaml_path)
    final_config = config["original_repo"]
    model_lang = ''
    model_quant = ''
    if 'default_language' in config:
        model_lang = config['default_language']
    if 'quantize' in config:
        model_quant = config['quantize']

    if model_lang != '' and model_quant != '':
        final_config += f'- ({flags[model_lang]}, {model_quant})'
    elif model_lang != '' and model_quant == '':
        final_config += f'- ({flags[model_lang]})'
    elif model_lang == '' and model_quant != '':
        final_config += f'- ({model_quant})'
    else:
        final_config = final_config
        
    return {f'{config["original_repo"]}': config["mlx-repo"]}, {f'{config["original_repo"]}': yaml_path}, {f'{final_config}': config["original_repo"]}, {f'{final_config}': config["mlx-repo"]}

def model_info():
    model_list = {}
    yml_list = {}
    final_cfg_list = {}
    mlx_config_list = {}
    directory_path = os.path.dirname(os.path.abspath(__file__))
    yaml_files = get_yaml_files(f'{directory_path}/configs')
    for file in yaml_files:
        model_dict, yml_path, final_cfg, mlx_config = process_yaml(file)
        model_list.update(model_dict)
        yml_list.update(yml_path)
        final_cfg_list.update(final_cfg)
        mlx_config_list.update(mlx_config)

    return model_list, yml_list, final_cfg_list, mlx_config_list


import gradio as gr
from mlx_lm import load, generate
from openai import OpenAI
import subprocess
from huggingface_hub import snapshot_download
from chat_with_mlx.models.utils import model_info
from chat_with_mlx.rag.utils import get_prompt
import os
import time
from langchain_community.embeddings import HuggingFaceEmbeddings
from langchain.text_splitter import RecursiveCharacterTextSplitter
from langchain_community.vectorstores import Chroma
from langchain_community.document_loaders import PyPDFLoader, Docx2txtLoader, TextLoader
from langchain_community.document_loaders import YoutubeLoader
import os
import argparse
from chat_with_mlx import __version__

os.environ['TOKENIZERS_PARALLELISM'] = "False"
SUPPORTED_LANG = ['default', 'English', 'Spanish', 'Chinese', 'Vietnamese', 'Japanese', 'Korean', 'Indian', 'Turkish', 'German', 'French', "Italian"]
openai_api_base = "http://127.0.0.1:8080/v1"
model_dicts, yml_path, cfg_list, mlx_config = model_info()
model_list = list(cfg_list.keys())
client = OpenAI(api_key='EMPTY',base_url=openai_api_base)
text_splitter = RecursiveCharacterTextSplitter(chunk_size=512, chunk_overlap=50)
emb = HuggingFaceEmbeddings(model_name='nomic-ai/nomic-embed-text-v1.5', model_kwargs={'trust_remote_code':True})
vectorstore = None

def load_model(model_name, lang):
    global process, rag_prompt, rag_his_prompt, sys_prompt, default_lang
    default_lang = 'default'
    prompts, sys_prompt = get_prompt(f'{yml_path[cfg_list[model_name]]}', lang)
    rag_prompt, rag_his_prompt = prompts[0], prompts[1]
    model_name_list = cfg_list[model_name].split('/')
    directory_path = os.path.dirname(os.path.abspath(__file__))
    local_model_dir = os.path.join(directory_path, 'models', 'download', model_name_list[1])
    
    if not os.path.exists(local_model_dir):
        snapshot_download(repo_id=mlx_config[model_name], local_dir=local_model_dir)
    
    command = [
        "python", "-m", "mlx_lm.server",
        "--model", local_model_dir
    ]
    
    try:
        process = subprocess.Popen(command, stdin=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        process.stdin.write('y\n')
        process.stdin.flush()
        return {model_status: f"Model Loaded"}
    except Exception as e:
        return {model_status: f"Exception occurred: {str(e)}"}

def kill_process():
    global process
    process.terminate()
    time.sleep(2)
    if process.poll() is None:  # Check if the process has indeed terminated
        process.kill()  # Force kill if still running

    print("Model Killed")
    return {model_status: 'Model Unloaded'}

def check_file_type(file_path):
    # Check for document file extensions
    if file_path.endswith('.pdf') or file_path.endswith('.txt') or file_path.endswith('.doc') or file_path.endswith('.docx'):
        return True
        # Check for YouTube link formats
    elif file_path.startswith('https://www.youtube.com/') or file_path.startswith('https://youtube.com/') or file_path.startswith('https://youtu.be/'):
        return True
    else:
        return False

def upload(files):
    supported = check_file_type(files)
    if supported:
        return {url: files, index_status: 'Not Done'}
    else:
        return {url: 'File type not supported', index_status: 'Not Done'}
    
def indexing(mode, url):
    global vectorstore
    
    try:

        if mode == 'Files (docx, pdf, txt)':
            if url.endswith('.pdf'):
                loader = PyPDFLoader(url)
            elif url.endswith('.docx'):
                loader = Docx2txtLoader(url)
            elif url.endswith('.txt'):
                loader = TextLoader(url)
            splits = loader.load_and_split(text_splitter)
        elif mode == 'YouTube (url)':
            loader = YoutubeLoader.from_youtube_url(url, 
                                                    add_video_info=False, language=['en', 'vi'])
            splits = loader.load_and_split(text_splitter)

        vectorstore = Chroma.from_documents(documents=splits, embedding=emb)
        return {index_status: 'Indexing Done'}
    except Exception as e:
        # Print the error message or return it as part of the response
        print(f"Error: {e}")  # This will print the error to the console or log
        return {'index_status': 'Indexing Error', 'error_message': str(e)}

def kill_index():
    global vectorstore
    vectorstore = None
    return {index_status: 'Indexing Undone'}

def build_rag_context(docs):
    context = ''
    for doc in docs:
        context += doc.page_content + '\n'

    return context

def chatbot(query, history, temp, max_tokens, freq_penalty, k_docs):
    global chat_history, sys_prompt

    
    if 'vectorstore' in globals() and vectorstore is not None:

        if len(history) == 0:
            chat_history = []
            if sys_prompt is not None:
                chat_history.append({'role': 'system', 'content': sys_prompt})
            docs = vectorstore.similarity_search(query, k=k_docs)
        else:
            history_str = ''
            for i, message in enumerate(history):
                history_str += f"User: {message[0]}\n"
                history_str += f"AI: {message[1]}\n"

            if sys_prompt is not None:
                chat_history.append({'role': 'system', 'content': sys_prompt})
            chat_history.append({'role': 'user', 'content': history_str})
            docs = vectorstore.similarity_search(history_str)


        context = build_rag_context(docs)

        if len(history) == 0:
            prompt = rag_prompt.format(context=context, question=query)
        else:
            prompt = rag_his_prompt.format(chat_history=history_str, context=context, question=query)
        messages = [{"role": "user", "content": prompt}]
    else:
        if len(history) == 0:
            chat_history = []
            if sys_prompt is not None:
                chat_history.append({'role': 'system', 'content': sys_prompt})
        else:
            chat_history = []
            if sys_prompt is not None:
                chat_history.append({'role': 'system', 'content': sys_prompt})
            for i, message in enumerate(history):
                chat_history.append({'role': 'user', 'content': message[0]})
                chat_history.append({'role': 'assistant', 'content': message[1]})
        chat_history.append({'role': 'user', 'content': query})
        messages = chat_history

    # Uncomment for debugging    
    # print(messages)
        
    response = client.chat.completions.create(
        model='gpt',
        messages=messages,
        temperature=temp,
        frequency_penalty=freq_penalty,
        max_tokens=max_tokens,
        stream=True,
    )
    stop = ['<|im_end|>', '<|endoftext|>']
    partial_message = ''
    for chunk in response:
        if len(chunk.choices) != 0:
            if chunk.choices[0].delta.content not in stop:
                partial_message = partial_message + chunk.choices[0].delta.content
            else:
                partial_message = partial_message + ''
            yield partial_message


with gr.Blocks(fill_height=True, theme=gr.themes.Soft()) as demo:

    model_name = gr.Dropdown(label='Model',info= 'Select your model', choices=sorted(model_list), interactive=True, render=False)
    temp_slider = gr.State(0.2)
    max_gen_token = gr.State(512)
    freq_penalty = gr.State(1.05)
    retrieve_docs = gr.State(3)
    language = gr.State('default')
    gr.ChatInterface(
        chatbot=gr.Chatbot(height=600,render=False),
        fn=chatbot,                        # Function to call on user input
        title="Chat with MLX🍎",    # Title of the web page
        description="Chat with your data using Apple MLX Backend",    # Description
        additional_inputs=[temp_slider, max_gen_token, freq_penalty, retrieve_docs]
    )
    with gr.Accordion("Advanced Setting", open=False):
        with gr.Row():
            with gr.Column(scale=2):
                temp_slider = gr.Slider(label='Temperature', value=0.2, minimum=0.0, maximum=1.0, step=0.05, interactive=True)
                max_gen_token = gr.Slider(label='Max Tokens', value=512, minimum=512, maximum=4096, step=256, interactive=True)
            with gr.Column(scale=2):
                freq_penalty = gr.Slider(label='Frequency Penalty', value=1.05, minimum=-2, maximum=2, step=0.05, interactive=True)
                retrieve_docs = gr.Slider(label='No. Retrieval Docs', value=3, minimum=1, maximum=10, step=1, interactive=True)

    with gr.Row():
        with gr.Column(scale=2):
            model_name.render()
            language = gr.Dropdown(label='Language', choices=sorted(SUPPORTED_LANG), value='default', interactive=True)
            btn1 = gr.Button("Load Model", variant='primary')
            btn3 = gr.Button("Unload Model", variant='stop')
        with gr.Column(scale=4):
            with gr.Row():
                with gr.Column(scale=9):
                    mode = gr.Dropdown(label='Dataset',info= 'Choose your dataset type', choices=['Files (docx, pdf, txt)', 'YouTube (url)'], scale=5)
                    url = gr.Textbox(label='URL', info='Enter your filepath (URL for Youtube)', interactive=True)
                    upload_button = gr.UploadButton(label='Upload File', variant='primary')
                    
                # data = gr.Textbox(visible=lambda mode: mode == 'YouTube')
                with gr.Column(scale=1):
                    model_status = gr.Textbox('Model Not Loaded', label='Model Status')
                    index_status = gr.Textbox("Not Index", label='Index Status')
                    btn1.click(load_model, inputs=[model_name, language], outputs=[model_status])
                    btn3.click(kill_process, outputs=[model_status])
                    upload_button.upload(upload, inputs=upload_button, outputs=[url, index_status])

                    index_button = gr.Button('Start Indexing', variant='primary')
                    index_button.click(indexing, inputs=[mode, url], outputs=[index_status])
                    stop_index_button = gr.Button('Stop Indexing')
                    stop_index_button.click(kill_index, outputs=[index_status])


def main():
    demo.launch(inbrowser=True)

if __name__ == "__main__":
    parser  = argparse.ArgumentParser(description="Chat with MLX \n"
                                    "Native RAG on MacOS and Apple Silicon with MLX 🧑‍💻")
    parser.add_argument("--version", action="version", 
                        version=f"Chat with MLX {__version__}")
    main()
